# -*- coding: utf-8 -*-
"""
oauthlib.oopenid.core
~~~~~~~~~~~~~~~~~~~~~~~

This module is an implementation of various logic needed
for consuming and providing OpenID Connect
"""
from __future__ import absolute_import, unicode_literals

from .pre_configured import Server
from .userinfo import UserInfoEndpoint
